@@ -7,7 +7,7 @@ require 5.010; # For named capture.
 
 use constant DEBUG_MRO => $ENV{DEBUG_YATT_MRO};
 
-use YATT::Lite::Core qw(Folder Template Part Widget Action);
+use YATT::Lite::Core qw(Folder Template Part Widget Action Entity);
 use YATT::Lite::Constants;
 
 # Naming convention:
@@ -101,7 +101,8 @@ use YATT::Lite::Constants;
   sub generate_widget {
     (my MY $self, my Widget $widget, my ($widget_name, $tmpl_path)) = @_;
     if ($widget->{cf_suppressed}) {
-      return "\n" x ($widget->{cf_endln} - $widget->{cf_startln});
+      # First line is alread used for package declaration.
+      return "\n" x ((($widget->{cf_endln} - 1) - $widget->{cf_startln}) - 2);
     }
     break_cgen();
     local $self->{curwidget} = $widget;
@@ -124,7 +125,7 @@ use YATT::Lite::Constants;
     (my MY $self, my Action $action) = @_;
     # XXX: 改行の調整が必要。
     my @src = ($self->sync_curline($action->{cf_startln})
-               , "sub $$action{cf_name} {");
+               , "sub do_$$action{cf_name} {");
     my $src = $self->{curtmpl}->source_substr
       ($action->{cf_bodypos}, $action->{cf_bodylen});
 
@@ -142,6 +143,32 @@ use YATT::Lite::Constants;
       + ($has_nl ? 1 : 0);
     (@src, $src, "}");
   }
+
+  # XXX: As you see, dup code.
+  # XXX: Should define common base class for Action and Entity.
+  sub generate_entity {
+    (my MY $self, my Entity $entity) = @_;
+    # XXX: 改行の調整が必要。
+    my @src = ($self->sync_curline($entity->{cf_startln})
+               , "sub entity_$$entity{cf_name} {");
+    my $src = $self->{curtmpl}->source_substr
+      ($entity->{cf_bodypos}, $entity->{cf_bodylen});
+
+    if (lexpand($entity->{arg_order})
+        or $src !~ m{^([\ \t\r\n]*)my\s*\([^;\)]+\)\s*=\s*\@_\s*;}) {
+      # If an entity has no arguments
+      # and its source doesn't start with my (...) = @_;,
+      # insert preamble and getargs.
+      push @src, q{ my ($this) = shift; my $CON = $this->CON;}
+        , $self->gen_getargs($entity, not $entity->{cf_implicit});
+    }
+
+    my $has_nl = $src =~ s/\r?\n\Z//;
+    $self->{curline} = $entity->{cf_bodyln} + numLines($src)
+      + ($has_nl ? 1 : 0);
+    (@src, $src, "}");
+  }
+
   #========================================
   sub gen_preamble {q{ my ($this, $CON) = splice @_, 0, 2;}}
   sub gen_getargs {
@@ -217,6 +244,13 @@ use YATT::Lite::Constants;
     while (@{$self->{curtoks}}) {
       my $node = shift @{$self->{curtoks}};
       unless (ref $node) {
+        # 次が element な時は、先行する indent を捨てる
+        if (@{$self->{curtoks}} and ref $self->{curtoks}[0]
+              and $self->{curtoks}[0][0] == TYPE_ELEMENT
+              and $node =~ /^[\ \t]+\z/
+          ) {
+          next;
+        }
 	# text node の末尾が改行で終わっている場合、 明示的に "\n" を生成する
 	my $has_nl = $node =~ s/\r?\n\Z//s;
 	push @queue, qtext($node) if $node ne ''; # 削ったら空になるかも。
@@ -242,6 +276,11 @@ use YATT::Lite::Constants;
 	push @queue, $expr;
 	$flush->() if $expr =~ /\n/;
       }
+      # Following can remove trailing newline from </yatt:foreach>,
+      # but it affects all of yatt tags.
+      # if (@{$self->{curtoks}} and $self->{curtoks}[0] eq "\n") {
+      #   shift @{$self->{curtoks}};
+      # }
     }
     $flush->();
     join " ", @result;
@@ -263,9 +302,10 @@ use YATT::Lite::Constants;
 	next;
       }
       # 許されるのは entity だけでは？ でもないか。 element 引数の時は、capture したいはず。
-      my $sub = $dispatch->[$node->[0]]
-	or die $self->generror("gen_as %s: Unknown node type: %d"
-			       , $type, $node->[0]);
+      my $sub = $dispatch->[$node->[0]] or do {
+        die $self->generror("gen_as %s: Unknown node type: %d"
+                            , $type, $node->[0]);
+      };
       my $expr = $sub->($self, $node);
       next unless defined $expr;
       if (ref $expr) {
@@ -316,9 +356,13 @@ use YATT::Lite::Constants;
   }
   sub text_from_element {
     (my MY $self, my $node) = @_;
-    &YATT::Lite::Breakpoint::breakpoint();
     my $call_ref = $self->from_element($node);
-    sprintf q{YATT::Lite::Util::captured {my ($CON) = @_; %s}}, $$call_ref;
+    sprintf q{(YATT::Lite::Util::captured {my ($CON) = @_; %s})}, $$call_ref;
+  }
+  sub text_from_pi {
+    (my MY $self, my $node) = @_;
+    my $statements = $self->from_pi($node);
+    sprintf q{(YATT::Lite::Util::captured {my ($CON) = @_; %s})}, $statements;
   }
 
   sub gen_call {
@@ -357,7 +401,7 @@ use YATT::Lite::Constants;
     my $add_arg = sub {
       my ($name) = @_;
       my $formal = $widget->{arg_dict}{$name} or do {
-	die $self->generror(q{Unknown arg '%s' in widget %s}, $name, $wname);
+	die $self->generror(q{Unknown arg '%s' in widget <%s>}, $name, $wname);
       };
       if (defined $argOrder[my $argno = $formal->argno]) {
 	die $self->generror(q{Duplicate arg '%s'}, $name);
@@ -371,9 +415,21 @@ use YATT::Lite::Constants;
       $self->sync_curline($_->[NODE_LNO]), ", ", $self->add_curline(do {
 	my $name = argName($_);
 	unless (defined $name) {
-	  defined($name = $widget->{arg_order}[$posArgs++])
-	    or die $self->generror("Too many args");
+	  defined($name = $widget->{arg_order}[$posArgs //= 0])
+	    or die $self->generror("Too many arguments for widget <%s>"); # This may not be called.
+
+          # Positional arguments should not be treated as body argument.
+          if ($widget->{arg_dict}{$name}->is_body_argument) {
+            die $self->generror(
+              "Too many arguments for widget <%s>: %s", $wname
+              , $self->{curtmpl}->source_region($primary->[$posArgs][NODE_BEGIN],
+                                                $primary->[$#$primary][NODE_END])
+            );
+          }
+
+          $posArgs++;
 	}
+
 	my $formal = $add_arg->($name);
 	unless (my $passThruVar = passThruVar($_)) {
 	  $self->as_cast_to($formal, argValue($_));
@@ -388,7 +444,7 @@ use YATT::Lite::Constants;
 	  # フラグ立てとして扱って良い型の場合。
 	  $v;
 	} else {
-	  die $self->generror(q{valueless arg '%s'}, $passThruVar);
+	  die $self->generror(q{argument '%s' requires value expression like '=...'}, $passThruVar);
 	}
       });
     } @$primary;
@@ -439,6 +495,9 @@ use YATT::Lite::Constants;
   sub as_varcall_delegate {
     (my MY $self, my ($var, $node)) = @_;
     my Widget $delegate = $var->widget;
+    unless (defined $delegate) {
+      Carp::croak "delegate target widget is empty!";
+    }
     $self->ensure_generated(perl => my Template $tmpl = $delegate->{cf_folder});
     my $that = $tmpl == $self->{curtmpl} ? '$this' : $tmpl->{cf_entns};
     \ sprintf(q{%s->render_%s($CON, %s)}
@@ -491,6 +550,9 @@ use YATT::Lite::Constants;
   }
   sub as_cast_to_code {
     (my MY $self, my ($var, $value)) = @_;
+    unless (ref $value eq 'ARRAY') {
+      die $self->generror(q{Invalid text expression for variable '%s': %s}, $var->varname, $value);
+    }
     local $self->{curtoks} = [@$value];
     my Widget $virtual = $var->widget;
     local $self->{scope} = $self->mkscope
@@ -503,7 +565,9 @@ use YATT::Lite::Constants;
   sub argName  {
     my ($arg, $skip) = @_;
     my $name = $$arg[NODE_PATH];
-    unless (wantarray and ref $name) {
+    if (array_of_array($name)) {
+      Carp::croak("namelist is given for argName()!");
+    } elsif (not (wantarray and ref $name)) {
       $name;
     } elsif (defined $skip) {
       @{$name}[$skip .. $#$name];
@@ -644,7 +708,8 @@ use YATT::Lite::Constants;
     (my MY $self, my ($escape_now)) = splice @_, 0, 2;
     return '' unless @_;
     local $self->{needs_escaping} = 0;
-    if (@_ == 1 and $_[0][0] eq 'call'
+    if (@_ == 1 and ($_[0][0] eq 'call'
+		       or $_[0][0] eq 'var' and $self->{cf_prefer_call_for_entity})
 	and my $macro = $self->can("entmacro_$_[0][1]")) {
       return $macro->($self, $_[0]);
     }
@@ -666,6 +731,25 @@ use YATT::Lite::Constants;
       sprintf(q{YATT::Lite::Util::escape(%s)}, $result);
     }
   }
+  # XXX: partial logic dup with ensure_entity_is_declared
+  sub find_entity {
+    (my MY $self, my ($name)) = @_;
+    my Template $tmpl = $self->{curtmpl};
+    $tmpl->{Item}{"entity\0$name"}
+      // $tmpl->{cf_entns}->can("entity_$name");
+  }
+  sub ensure_entity_is_declared {
+    (my MY $self, my ($name)) = @_;
+    my Template $tmpl = $self->{curtmpl};
+    if ($tmpl->{Item}{"entity\0$name"}) {
+      # Found embedded entity definition.
+      return;
+    }
+    unless ($tmpl->{cf_entns}->can("entity_$name")) {
+      die $self->generror(q!No such entity in namespace "%s": %s!
+			  , $tmpl->{cf_entns}, $name);
+    }
+  }
   sub gen_entlist {
     (my MY $self, my ($escape_now)) = splice @_, 0, 2;
     my @list = map {
@@ -675,12 +759,16 @@ use YATT::Lite::Constants;
   }
   sub as_expr_var {
     (my MY $self, my ($esc_later, $name)) = @_;
-    my $var = $self->find_var($name)
-      or die $self->generror(q{No such variable '%s'}, $name);
-    if (my $sub = $self->can("as_expr_var_" . $var->type->[0])) {
-      $sub->($self, $esc_later, $var, $name);
+    if (my $var = $self->find_var($name)) {
+      if (my $sub = $self->can("as_expr_var_" . $var->type->[0])) {
+	$sub->($self, $esc_later, $var, $name);
+      } else {
+	$self->as_lvalue($var);
+      }
+    } elsif ($self->find_entity($name)) {
+      $self->gen_entcall($name);
     } else {
-      $self->as_lvalue($var);
+      die $self->generror(q{No such variable '%s'}, $name);
     }
   }
   sub as_expr_var_html {
@@ -705,13 +793,14 @@ use YATT::Lite::Constants;
       return $self->as_expr_call_var($var, $name, @_);
     }
 
-    my Template $tmpl = $self->{curtmpl};
-    unless ($tmpl->{cf_entns}->can("entity_$name")) {
-      die $self->generror(q!No such entity in namespace "%s": %s!
-			  , $tmpl->{cf_entns}, $name);
-    }
+    $self->ensure_entity_is_declared($name);
+
+    $self->gen_entcall($name, @_);
+  }
+  sub gen_entcall {
+    (my MY $self, my ($name, @rest)) = @_;
     my $call = sprintf '$this->entity_%s(%s)', $name
-      , scalar $self->gen_entlist(undef, @_);
+      , scalar $self->gen_entlist(undef, @rest);
     $call;
   }
   sub as_expr_call_var {
@@ -758,7 +847,9 @@ use YATT::Lite::Constants;
   }
   sub as_expr_prop {
     (my MY $self, my ($esc_later, $name)) = @_;
-    if ($name =~ /^\w+$/) {
+    if ($self->{cf_prefer_call_for_entity}) {
+      $name
+    } elsif ($name =~ /^\w+$/) {
       "{$name}"
     } else {
       '{'.qtext($name).'}';
@@ -786,10 +877,16 @@ sub feed_arg_spec {
     my ($name, @ext) = argName($arg); # XXX: <yatt:my var:type=value /> は？
     unless (defined $name) {
       $name = $arg_order->[$nth++]
-	or die $trans->generror($arg, "Too many args");
+	or die $trans->generror("Too many args");
     }
     defined (my $argno = $arg_dict->{$name})
-      or die $trans->generror($arg, "Unknown arg '%s'", $name);
+      or die $trans->generror("Unknown arg '%s'", $name);
+
+    if (defined (my $prevNode = $_[$argno])) {
+      die $trans->generror("You may forgot '=' between %s and %s"
+                           , $prevNode->[NODE_PATH]
+                           , $trans->{curtmpl}->node_outer_source($arg));
+    }
 
     $_[$argno] = $arg;
     $found++;
@@ -817,22 +914,27 @@ sub feed_arg_spec {
     # いかん、 cond を生成するなら、body も生成しておかないと、行番号が困る。
 
     foreach my $arg (lexpand($foot)) {
-      if ($arg->[NODE_PATH][-1] eq 'else') {
-	$self->feed_arg_spec($arg->[NODE_ATTLIST], \%args, \@args
-			     , my ($if, $unless));
-	my ($fmt, $guard) = do {
-	  if ($if) { (q{elsif (%s) }, $if->[NODE_VALUE]) }
-	  elsif ($unless) { (q{elsif (not %s) }, $unless->[NODE_VALUE]) }
-	  else { (q{else }, undef) }
-	};
-	push @arms, [$fmt, $guard, lexpand($arg->[NODE_VALUE])]
-      } else {
-	push @{$arms[-1]}, lexpand($arg->[NODE_VALUE]);
+      (undef, my $kw) = @{$arg->[NODE_PATH]};
+      unless ($kw eq 'else') {
+        die $self->generror("Unknown option for <%s>: %s"
+                            , join(":", @$path), $kw);
       }
+      $self->feed_arg_spec($arg->[NODE_ATTLIST], \%args, \@args
+                           , my ($if, $unless));
+      my ($fmt, $guard) = do {
+        if ($if) {
+          (q{elsif (%s) }, $if->[NODE_VALUE]);
+        } elsif ($unless) {
+          (q{elsif (not %s) }, $unless->[NODE_VALUE]);
+        } else {
+          (q{else }, undef);
+        }
+      };
+      push @arms, [$fmt, $guard, lexpand($arg->[NODE_VALUE])]
     }
-    local $self->{scope} = $self->mkscope({}, $self->{scope});
     my @expr = map {
       my ($fmt, $guard, @body) = @$_;
+      local $self->{scope} = $self->mkscope({}, $self->{scope});
       local $self->{curtoks} = [@body];
       (defined $guard
        ? sprintf($fmt, join "", $self->as_list(lexpand($guard))) : $fmt)
@@ -842,13 +944,23 @@ sub feed_arg_spec {
   }
 }
 
+sub is_spread {
+  ref $_[0] eq 'ARRAY'
+    && @{$_[0]} == 4
+    && $_[0][0] eq ''
+    && $_[0][1] eq ''
+    && $_[0][2] eq ''
+}
+sub take_spread_name {
+  $_[0]->[3];
+}
+
 {
   sub macro_my {
     (my MY $self, my $node) = @_;
-    my ($path, $body, $primary, $head, $foot) = nx($node);
-
+    my ($path, $body, $maybeWrappedAttlist, $head, $foot) = nx($node);
     my $has_body = $body && @$body ? 1 : 0;
-    my $adder = sub {
+    my $simple_adder = sub {
       my ($default_type, $arg, $valNode, $skip) = @_;
       my ($name, $typename) = argName($arg, $skip);
       if (my $oldvar = $self->find_var($name)) {
@@ -867,9 +979,55 @@ sub feed_arg_spec {
 	# typename == source の時が問題だ。
 	my $expr = 'my '.$self->as_lvalue($var);
 	my $value = argValue($valNode);
-	$expr .= $value ? (' = '.$self->as_cast_to($var, $value)) : ';';
+	$expr .= defined $value ? (' = '.$self->as_cast_to($var, $value)) : ';';
       }
     };
+    my $adder = sub {
+      my ($default_type, $arg, $valNode, $skip) = @_;
+      if (array_of_array($arg->[NODE_PATH])) {
+        my (@pre, @main);
+        foreach my $nameSpec (@{$arg->[NODE_PATH]}) {
+          my $is_spread = is_spread($nameSpec->[NODE_PATH]);
+          my ($name, $typename) = do {
+            if ($is_spread) {
+              (take_spread_name($nameSpec->[NODE_PATH]), 'list')
+            } else {
+              argName($nameSpec, $skip);
+            }
+          };
+
+          if (my $oldvar = $self->find_var($name)) {
+            die $self->generror("Conflicting variable '%s'"
+                                ." (previously defined at line %s)"
+                                , $name, $oldvar->lineno // '(unknown)');
+          }
+          $typename ||= $default_type;
+          if (my $sub = $self->can("_macro_my_$typename")) {
+            ...
+          }
+          my $var = $self->{scope}[0]{$name}
+            = $self->mkvar_at(undef, $typename, $name)
+            or die $self->generror("Unknown type '%s' for variable '%s'"
+                                   , $typename, $name);
+
+            if (@pre) {
+              die $self->generror("Multiple spread op is prohibited");
+            }
+          if ($is_spread) {
+            push @pre, 'my '.$self->as_lvalue($var).' = []';
+            push @main, '@'.$self->as_lvalue($var);
+          } else {
+            push @main, 'my '.$self->as_lvalue($var);
+          }
+        }
+        my $main = sprintf(q|(%s) = (%s)|, join(", ", @main)
+                           , join("", $self->as_list(lexpand(argValue($valNode)))));
+        (@pre, $main);
+      } else {
+        $simple_adder->(@_)
+      }
+    };
+    my $primary = $self->node_unwrap_attlist($maybeWrappedAttlist);
     my @assign;
     foreach my $arg (@{$primary}[0 .. $#$primary-$has_body]) {
       push @assign, $adder->(text => $arg, $arg);
@@ -877,8 +1035,8 @@ sub feed_arg_spec {
     if ($has_body) {
       my $arg = $primary->[-1];
       # XXX: ここは統合できるはず。ただし、NESTED の時に name が無いことを確認すべき。
-      if ($$arg[NODE_TYPE] == TYPE_ATT_NESTED) {
-	foreach my $each (nx($arg, 1)) {
+      if ($arg->[NODE_TYPE] == TYPE_ATT_NESTED) {
+	foreach my $each (@{$arg->[NODE_BODY]}) {
 	  push @assign, $adder->(html => $each, $body);
 	}
       } else {
@@ -925,7 +1083,7 @@ sub feed_arg_spec {
     my ($path, $body, $primary, $head, $foot) = nx($node);
     $self->feed_arg_spec($primary, \%args, \@args
 			 , my ($list, $my, $nth))
-      or die $self->generror("Not enough arguments!");
+      or die $self->generror("Not enough arguments for <yatt:foreach>!");
 
     my ($prologue, $continue, $epilogue) = ('', '', '');
 
@@ -974,13 +1132,31 @@ sub feed_arg_spec {
 	}
 	'@'.$self->as_lvalue($found_var);
       } else {
-	die $self->generror("Unknown list=");
+	die die $self->generror("Unknown variable for foreach list: '%s'", $passThruVarName);
       }
     };
 
     local $self->{curtoks} = [@{argValue($body)}];
+
+    # <yatt:foreach ..>\n
+    #                  ↑This newline should be removed.
+    my $statements = "{";
+    if (my $nl = $self->cut_next_nl) {
+      $statements .= $nl;
+    }
+
+    # ___</yatt:foreach>
+    # ↑ This indent is messy too.
+    if (@{$self->{curtoks}} and $self->{curtoks}[-1] =~ /^[\ \t]+\z/) {
+      pop @{$self->{curtoks}};
+    }
+
+    #    ...\n         ← not to remove this newline.
+    # </yatt:foreach>
+    local $self->{no_last_newline} = 0;
+
     local $self->{scope} = $self->mkscope(\%local, $self->{scope});
-    my $statements = '{'.$self->as_print('}');
+    $statements .= $self->as_print('}');
 
     if ($opts and $opts->{fragment}) {
       ($fmt, $loopvar, $listexpr, $statements
@@ -992,9 +1168,112 @@ sub feed_arg_spec {
   }
 }
 
-sub entx {
-  my ($node) = @_;
-  @{$node}[2..$#$node];
+{
+  MY->make_arg_spec(\ my %args, \ my @args, qw(if unless local));
+  sub macro_return {
+    (my MY $self, my $node) = @_;
+    my ($path, $body, $primary, $head, $foot) = nx($node);
+
+    if ($foot || $head) {
+      die $self->generror("Unsupported syntax: foot");
+    }
+
+    $self->feed_arg_spec($primary, \%args, \@args
+                         , my ($if, $unless, $local));
+
+    my ($fmt, $guard) = do {
+      if ($if || $unless) {
+        # conditional return
+        # if ()
+        my ($kw, $cond) = do {
+          if ($if) { (if => $if) }
+          elsif ($unless) { (unless => $unless) }
+          else { die "??" }
+        };
+        ("$kw (%s) ", $cond->[NODE_VALUE]);
+      } else {
+        ();
+      }
+    };
+
+    my ($begin, $end) = do {
+      if ($local) {
+        ('{', ' return}');
+      } else {
+        ('{YATT::Lite::Util::rewind($CON); ', ' die \"DONE"}');
+      }
+    };
+
+    my $expr = do {
+      local $self->{scope} = $self->mkscope({}, $self->{scope});
+      local $self->{curtoks} = [lexpand($body->[NODE_VALUE])];
+      (defined $guard
+       ? sprintf($fmt, join "", $self->as_list(lexpand($guard))) : '')
+	.$begin.$self->cut_next_nl.$self->as_print($end);
+    };
+
+    \ $expr;
+  }
+}
+
+# A skeleton for new macro.
+# {
+#   MY->make_arg_spec(\ my %args, \ my @args, qw(if unless));
+#   sub macro_return {
+#     (my MY $self, my $node) = @_;
+#     my ($path, $body, $primary, $head, $foot) = nx($node);
+#
+#     \ ($self->dump_as_comment_line($self->node_as_hash($node)));
+#   }
+# }
+
+sub dump_as_comment_line {
+  (my MY $self, my $node) = @_;
+  my $str = terse_dump($node);
+  $str =~ s/^/# /mg;
+  $str."\n";
+}
+
+# A skelton for new entmacro
+# sub entmacro_XXX {
+#   (my MY $self, my $node) = @_;
+#   sprintf(q{YATT::Lite::Util::escape(%s)}, terse_dump($node));
+# }
+
+sub entmacro_scalar {
+  (my MY $self, my $node) = @_;
+  my (@expr) = $self->gen_entlist(undef, entx($node));
+  sprintf q|scalar(%s)|, join "", @expr;
+}
+
+sub entmacro_not {
+  (my MY $self, my $node) = @_;
+  my (@expr) = $self->gen_entlist(undef, entx($node));
+  sprintf q|(not(%s))|, join "", @expr;
+}
+
+sub entmacro_and {
+  (my MY $self, my $node) = @_;
+  my (@expr) = $self->gen_entlist(undef, entx($node));
+  "(".join(" and ", map {"($_)"} @expr).")";
+}
+
+sub entmacro_or {
+  (my MY $self, my $node) = @_;
+  my (@expr) = $self->gen_entlist(undef, entx($node));
+  "(".join(" or ", map {"($_)"} @expr).")";
+}
+
+sub entmacro_undef {
+  (my MY $self, my $node) = @_;
+  my (@expr) = $self->gen_entlist(undef, entx($node));
+  sprintf q|(undef(%s))|, join "", @expr;
+}
+
+sub entmacro_with_ignoring_die {
+  (my MY $self, my $node) = @_;
+  my (@expr) = $self->gen_entlist(undef, entx($node));
+  sprintf q|($this->YATT->with_ignoring_die(sub {%s}))|, join "", @expr;
 }
 
 sub entmacro_if {
@@ -1057,6 +1336,22 @@ sub entmacro_dispatch_one {
     , $prefix, $nargs, join(", ", @list);
 }
 
+sub entmacro___WIDGET__ {
+  (my MY $self, my $node) = @_;
+  my Widget $widget = $self->{curwidget};
+  qtext($widget->{cf_name});
+}
+
+sub entmacro_show_expr {
+  (my MY $self, my $node) = @_;
+  require YATT::Lite::LRXML::FormatEntpath;
+  my (@pipe) = entx($node);
+  \ sprintf q{$this->YATT->render_into($CON, %s, [%s, %s])}
+    , qtext('show_expr')
+    , qtext(YATT::Lite::LRXML::FormatEntpath::format_entpath(@pipe))
+    , join(", ", $self->gen_entpath(undef, @pipe));
+}
+
 use YATT::Lite::Breakpoint qw(break_load_cgen break_cgen);
 break_load_cgen();
 
