@@ -13,13 +13,23 @@ use YATT::Lite::Util::Enum
 		  ENTITY PI ELEMENT
 		  ATTRIBUTE=ATT_NAMEONLY ATT_BARENAME ATT_TEXT ATT_NESTED
 		  ATT_MACRO=DECL_ENTITY)]
-   , NODE_ => [qw(TYPE BEGIN END LNO PATH REST=VALUE=BODY ATTLIST
-		  AELEM_HEAD AELEM_FOOT BODY_BEGIN BODY_END)]
+   , NODE_ => [qw(TYPE BEGIN END LNO
+                  SYM_END
+                  BODY_BEGIN BODY_END
+                  PATH REST=VALUE=BODY ATTLIST
+		  AELEM_HEAD AELEM_FOOT)]
    # node item
    # BODY が必ず配列になるが、代わりに @attlist は配列不要に。 空の [] を pad しなくて済む
    # XXX: <:yatt:else /> とかもあったじゃん！
+
+   , ENT_ => [qw(TYPE KEY REST=VALUE=BODY)]
   );
 
+# Not worked. why?
+# BEGIN {
+#   push our @EXPORT_OK, qw/*TYPE_ *NODE_/;
+# }
+
 sub cut_first (&@) {
   my ($code, $list) = @_;
   local $_;
@@ -36,10 +46,6 @@ sub cut_first_att {
   cut_first {$_->[NODE_TYPE] >= TYPE_ATTRIBUTE} $list;
 }
 
-# list expand if nested.
-sub lxnest {
-  ref $_[0][0] ? @{$_[0]} : $_[0]
-}
 # node expand.
 sub nx {
   @{$_[0]}[(NODE_PATH + ($_[1] // 0)) .. $#{$_[0]}];
@@ -59,14 +65,17 @@ sub qqvalue ($) {
   'q'.qtext($_[0]);
 }
 
-sub node_path {
-  my ($self, $node) = @_;
-  $node->[NODE_PATH];
-}
+sub node_type    { $_[1]->[NODE_TYPE] }
+sub node_path    { $_[1]->[NODE_PATH] }
+sub node_attlist { $_[1]->[NODE_ATTLIST] }
 
-sub node_attlist {
-  my ($self, $node) = @_;
-  $node->[NODE_ATTLIST];
+sub node_has_name_list {
+  array_of_array($_[1][NODE_PATH]);
+}
+sub array_of_array {
+  my ($path) = @_;
+  ref $path eq 'ARRAY'
+    && ref $path->[0] eq 'ARRAY'
 }
 
 sub node_body {
@@ -88,6 +97,19 @@ sub node_body_slot {
   }
 }
 
+sub node_unwrap_attlist {
+  my ($self, $maybeWrappedAttlist) = @_;
+  if ($maybeWrappedAttlist
+      and @$maybeWrappedAttlist == 1
+      and ref $maybeWrappedAttlist->[0] eq 'ARRAY'
+      and @{$maybeWrappedAttlist->[0]} == 1
+      and ref $maybeWrappedAttlist->[0][0] eq 'ARRAY') {
+    $maybeWrappedAttlist->[0]
+  } else {
+    $maybeWrappedAttlist;
+  }
+}
+
 sub node_value {
   my ($self, $node) = @_;
   wantarray ? YATT::Lite::Util::lexpand($node->[NODE_VALUE])
@@ -99,6 +121,34 @@ sub node_extract {
   nx($node, @_);
 }
 
+sub node_as_hash {
+  my ($self, $node) = @_;
+
+  my $hash;
+  foreach my $k (qw(TYPE
+                    PATH BODY ATTLIST
+                    AELEM_HEAD AELEM_FOOT)) {
+    my $i = __PACKAGE__->can("NODE_$k")->();
+    next if $#$node <= $i;
+    $hash->{lc($k)} = $node->[$i] if defined $node->[$i];
+  }
+  $hash;
+}
+
+#========================================
+# Entity entpath related wrappers.
+# Each element in entpath has its own structure.
+
+sub entx {
+  my ($ent) = @_;
+  @{$ent}[ENT_BODY..$#$ent];
+}
+
+# list expand if nested.
+sub lxnest {
+  ref $_[0][0] ? @{$_[0]} : $_[0]
+}
+
 #========================================
 my $symtab = YATT::Lite::Util::symtab(__PACKAGE__);
 our @EXPORT = grep {*{$symtab->{$_}}{CODE}} keys %$symtab;
