@@ -27,13 +27,19 @@ use YATT::Lite::MFields
 ([cf_namespace =>
   (doc => "namespace prefix for yatt. (default: [yatt, perl])")]
 
- , [cf_doc_root =>
+ , ["cf_^doc_root" =>
     (doc => "Primary template directory")]
 
- , [cf_app_base =>
-    (doc => "Base dir for this siteapp")]
+ # Note: Don't confuse 'cf_app_rootname' below with 'cf_app_root'.
+ # 'cf_app_root' is defined in YATT::Lite::Partial::AppPath.
+ #
+ , [cf_app_rootname =>
+    (doc => "rootname() of app.psgi. Used to find app.site_config.yml")]
 
- , [cf_site_prefix =>
+ , ["cf_^app_base" =>
+    (doc => "Base template dir for all DirApps")]
+
+ , ["cf_^site_prefix" =>
     (doc => "Location prefix for this siteapp")]
 
  , [cf_index_name =>
@@ -56,7 +62,10 @@ use YATT::Lite::MFields
      ." Also this is used to decode incoming request parameters and PATH_INFO."
      ." (default: 'utf-8')")]
 
- , [cf_offline =>
+ , [cf_render_as_bytes =>
+    (doc => "Force render() to return raw bytes. (default: false)")]
+
+ , ["cf_^offline" =>
     (doc => "Whether header should be emitted or not.")]
 
  , [cf_binary_config   =>
@@ -107,17 +116,32 @@ use YATT::Lite::MFields
        _my_psgi_app
 
        cf_match_argsroute_first
-     /);
+       /
+ , [cf_stash_unknown_params_to => 
+    (doc => "Stash unknown foreign parameters into this name. Set to 'yatt.unknown_params' when PLACK_ENV is *not* development.")]
+ , [cf_body_argument =>
+    (doc => "Name of 'body' argument. (default: body)")]
+ , [cf_body_argument_type =>
+    (doc => "Type of 'body' argument. (default: code)")]
+ , [cf_prefer_call_for_entity =>
+    (doc => ":name is interpreted as call if appropriate")]
+);
 
-use YATT::Lite::Util::AsBase;
+use YATT::Lite::Util::AsBase qw/-as_base import/;
 use YATT::Lite::Util qw/lexpand globref untaint_any ckrequire dofile_in
 			lookup_dir fields_hash
 			lookup_path
 			secure_text_plain
 			psgi_error
+                        psgi_text
 			globref_default
 			define_const
 			terse_dump
+                        psgi_dump
+                        raise_psgi_dump
+                        raise_response
+                        trimleft_length
+                        get_entity_symbol
 		       /;
 
 use YATT::Lite::XHF ();
@@ -126,7 +150,7 @@ use YATT::Lite::Partial::ErrorReporter;
 use YATT::Lite::Partial::AppPath;
 
 use YATT::Lite qw/Entity *SYS *YATT *CON/;
-
+our @EXPORT_OK = qw/*CON/;
 
 use YATT::Lite::Util::CycleDetector qw/Visits/;
 
@@ -165,6 +189,22 @@ sub configure_offline {
 
 #========================================
 
+#
+# $class->create_factory_class($fresh_classname)
+# Make $fresh_classname inherit Factory.
+# Mainly for tests which create many factory classes in single run.
+#
+sub create_factory_class {
+  my ($pack, $factory_class) = @_;
+
+  define_const(globref($factory_class, 'default_app_ns')
+               , $factory_class."::YATT");
+
+  $pack->_import_as_base($factory_class);
+
+  $factory_class;
+}
+
 sub load_factory_for_psgi {
   my ($pack, $psgi, %default) = @_;
   unless (defined $psgi) {
@@ -175,6 +215,8 @@ sub load_factory_for_psgi {
   }
   (my $app_rootname = $pack->rel2abs($psgi)) =~ s/\.psgi$//;
 
+  $default{app_rootname} //= $app_rootname;
+
   #
   # Assume app_root is safe.
   #
@@ -187,18 +229,61 @@ sub load_factory_for_psgi {
   if (-d "$app_root/ytmpl") {
     $default{app_base} ||= '@ytmpl';
   }
-  if (my (@cf) = map {
-    my $cf = "$app_rootname.$_";
-    -e $cf ? $cf : ()
-  } $pack->default_config_filetypes) {
-    croak "Multiple configuration files!: @cf" if @cf > 1;
-    $pack->_with_loading_file($cf[0], sub {
-				$pack->new(app_root => $app_root, %default
-					   , $pack->read_file($cf[0]));
-			      })
-  } else {
-    $pack->new(app_root => $app_root, %default);
+
+  my $env = delete $default{environment};
+
+  my (@cf) = $pack->list_config_files($app_rootname);
+
+  if (@cf and $env) {
+    croak "Can't use environment and @cf at once!";
+  } elsif (@cf > 1) {
+    croak "Multiple configuration files!: @cf";
   }
+
+  if ($env) {
+    my $config = $pack->config_for_env($app_root, $env);
+    return $pack->_with_loading_file($config, sub {
+      $pack->new(app_root => $app_root, %default
+                 , $pack->load_config($config));
+    });
+  }
+
+  my MY $self = do {
+    if (@cf) {
+      $pack->_with_loading_file($cf[0], sub {
+                                  $pack->new(app_root => $app_root, %default
+                                               , $pack->read_file($cf[0]));
+                                })
+    } else {
+      $pack->new(app_root => $app_root, %default);
+    }
+  };
+
+  unless ($self->{__after_new_is_called__}) {
+    Carp::croak("after_new is not called correctly!");
+  }
+
+  $self;
+}
+
+#
+# Load Amon2 style config.pl
+#
+sub config_for_env {
+  my ($pack, $app_root, $environment) = @_;
+  "$app_root/config/$environment.pl";
+}
+
+sub load_config {
+  my ($pack, $cf) = @_;
+  my $config = dofile_in($pack, $cf);
+  unless (defined $config) {
+    croak "config script '$cf' returned undef!";
+  }
+  unless (ref $config eq 'HASH') {
+    croak "config script '$cf' doesn't return HASH! $config";
+  }
+  wantarray ? %$config : $config;
 }
 
 #========================================
@@ -210,7 +295,7 @@ sub n_destroyed {$_n_destroyed}
   our %sub2self;
   DESTROY {
     (my MY $self) = @_;
-    print STDERR "DESTROY $self\n" if DEBUG_FACTORY;
+    print STDERR "# DESTROY $self\n" if DEBUG_FACTORY;
     delete $self->{_my_psgi_app};
     if (my $outer = delete $self->{_outer_psgi_app}) {
       delete $sub2self{$outer};
@@ -268,7 +353,17 @@ sub n_destroyed {$_n_destroyed}
       die "Unknown load result from: $fn";
     }
   }
-  sub prepare_app { return }
+  sub prepare_app {
+    (my MY $self) = shift;
+    $self->maybe::next::method(@_);
+  }
+  sub prepare_deployment {
+    (my MY $self) = shift;
+    $self->maybe::next::method(@_);
+    $self->{cf_stash_unknown_params_to}
+      //= $self->default_stash_unknown_params_to;
+  }
+  sub finalize_response { shift->maybe::next::method(@_) }
 
   our $load_count;
   sub sandbox_dofile {
@@ -315,6 +410,23 @@ sub find_factory_script {
   return;
 }
 
+#========================================
+# `use Factory -as_base` in app.psgi will come here.
+#
+sub _import_as_base {
+  my ($myPack, $callpack) = @_;
+  $myPack->SUPER::_import_as_base($callpack);
+  # To define EntNS in $callpack and set $myPack to @{$callpack::ISA}.
+  $myPack->default_default_app->define_Entity(undef, $callpack);
+}
+
+# Just to allow writing `use Factory -Entity;` in your app.psgi.
+# Not actually required.
+sub _import_Entity {
+  my ($myPack, $callpack) = @_;
+  $myPack->default_default_app->define_Entity(undef, $callpack);
+}
+
 #========================================
 
 sub new {
@@ -360,6 +472,7 @@ sub after_new {
       //= ($self->{cf_output_encoding} || $self->default_header_charset);
     $self->{cf_output_encoding}
       //= $self->compat_default_output_encoding;
+    $self->{cf_render_as_bytes} = 1;
   } else {
     $self->{cf_header_charset}
       //= ($self->{cf_output_encoding} // $self->default_header_charset);
@@ -368,6 +481,16 @@ sub after_new {
     $self->{cf_output_encoding} //= $self->default_output_encoding;
   }
   $self->{cf_use_subpath} //= 1;
+
+  $self->{cf_always_refresh_deps} //= $self->default_always_refresh_deps;
+
+  $self->{cf_body_argument} //= $self->default_body_argument;
+  $self->{cf_body_argument_type} //= $self->default_body_argument_type;
+
+  # prepare_app is too late to set delegated params
+  if (($ENV{PLACK_ENV} // '') ne 'development') {
+    $self->prepare_deployment;
+  }
 }
 
 sub compat_default_output_encoding { '' }
@@ -377,6 +500,11 @@ sub default_tmpl_encoding   { 'utf-8' }
 sub default_index_name { 'index' }
 sub default_ext_public {'yatt'}
 sub default_ext_private {'ytmpl'}
+sub default_stash_unknown_params_to {'yatt.unknown_params'}
+sub default_always_refresh_deps { 1 }
+
+sub default_body_argument { 'body' }
+sub default_body_argument_type { 'code' }
 
 sub _after_after_new {
   (my MY $self) = @_;
@@ -417,11 +545,15 @@ sub _after_after_new {
 sub render {
   my MY $self = shift;
   my $raw_bytes = $self->render_encoded(@_);
-  decode(utf8 => $raw_bytes);
+  if ($self->{cf_render_as_bytes}) {
+    $raw_bytes;
+  } else {
+    decode(utf8 => $raw_bytes);
+  }
 }
 
-sub render_encoded {
-  (my MY $self, my ($reqrec, $args, @opts)) = @_;
+sub parse_path_info {
+  (my MY $self, my ($reqrec)) = @_;
   # [$path_info, $subpage, $action]
   my ($path_info, @rest) = ref $reqrec ? @$reqrec : $reqrec;
 
@@ -430,35 +562,99 @@ sub render_encoded {
   my ($tmpldir, $loc, $file, $trailer, $is_index)
     = my @pi = $self->lookup_split_path_info($path_info);
   unless (@pi) {
-    die "No such location: $path_info";
+    return;
   }
 
+  ($path_info, $tmpldir, $loc,
+   (@rest ? [$file, @rest] : $file),
+   , $self->pi_to_connection_quad(\@pi)
+   );
+}
+
+sub prepare_processing_context {
+  (my MY $self, my ($reqrec, $args)) = @_;
+
+  my ($path_info, $tmpldir, $loc, $widgetSpec, @rest)
+    = $self->parse_path_info($reqrec) or do {
+      die "No such location: ".terse_dump($reqrec);
+    };
+
   my $dh = $self->get_lochandler(map {untaint_any($_)} $loc, $tmpldir) or do {
     die "No such directory: $path_info";
   };
 
-  my $con = $self->make_simple_connection
+  my $con = $self->make_connection
   (
-    \@pi, yatt => $dh, noheader => 1, path_info => $path_info
+    undef, @rest,
+    , yatt => $dh, noheader => 1, path_info => $path_info,
+    , encoding => $self->{cf_output_encoding}
     , $self->make_debug_params($reqrec, $args)
   );
 
+  ($dh, $con, $widgetSpec);
+}
+
+sub render_encoded {
+  (my MY $self, my ($reqrec, $args, @opts)) = @_;
+
+  my ($dh, $con, $widgetSpec) = $self->prepare_processing_context($reqrec, $args);
+
   $self->invoke_dirhandler
   (
-    $dh, $con
+    $dh,
    , render_into => $con
-   , @rest ? [$file, @rest] : $file
+   , $widgetSpec
    , $args, @opts
   );
 
   $con->buffer;
 }
 
+sub render_into {
+  (my MY $self, my ($con, $reqrec, $args, @opts)) = @_;
+
+  my ($path_info, $tmpldir, $loc, $widgetSpec, @rest)
+    = $self->parse_path_info($reqrec) or do {
+      die "No such location: ".terse_dump($reqrec);
+    };
+
+  my $dh = $self->get_lochandler(map {untaint_any($_)} $loc, $tmpldir) or do {
+    die "No such directory: $path_info";
+  };
+
+  $con->configure(yatt => $dh);
+
+  $self->invoke_dirhandler
+  (
+    $dh,
+   , render_into => $con
+   , $widgetSpec
+   , $args, @opts
+  );
+}
+
+sub make_connection_for {
+  (my MY $self, my ($reqrec, $args, @other)) = @_;
+
+  my ($path_info, $tmpldir, $loc, $widgetSpec, @rest)
+    = $self->parse_path_info($reqrec);
+
+  $self->make_connection
+  (
+    undef, @rest,
+    , noheader => 1, path_info => $path_info,
+    , encoding => $self->{cf_output_encoding},
+    , $self->make_debug_params($reqrec, $args),
+    , @other,
+  );
+}
+
 sub lookup_split_path_info {
   (my MY $self, my $path_info) = @_;
   lookup_path($path_info
 	      , $self->{tmpldirs}
-	      , $self->{cf_index_name}, ".$self->{cf_ext_public}"
+	      , $self->{cf_index_name}
+              , $self->{cf_ext_public}
 	      , $self->{cf_use_subpath});
 }
 
@@ -495,7 +691,8 @@ sub mount_psgi {
   }
   my $dict = $self->{loc2psgi_dict} //= +{};
   my $key = K_MOUNT_MATCH() . (keys %$dict);
-  $dict->{$path_prefix} = [$key => $path_prefix => $app, @opts];
+  (my $strip = $path_prefix) =~ s,/\z,,;
+  $dict->{$path_prefix} = [$key => $strip => $app, @opts];
 
   undef $self->{loc2psgi_re};
 
@@ -506,12 +703,13 @@ sub mount_psgi {
 sub rebuild_psgi_mount {
   (my MY $self) = @_;
   my @re;
-  foreach my $path_prefix (keys %{$self->{loc2psgi_dict} //= +{}}) {
+  foreach my $path_prefix (sort {length($b) <=> length($a)}
+                             keys %{$self->{loc2psgi_dict} //= +{}}) {
     my ($key, undef, $app) = @{$self->{loc2psgi_dict}{$path_prefix}};
     push @re, qr{(?<$key>$path_prefix)};
   }
   my $all = join("|", @re);
-  $self->{loc2psgi_re} = qr{^(?:$all)(?:/|$)};
+  $self->{loc2psgi_re} = qr{^(?:$all)};
 }
 
 sub psgi_file_app {
@@ -524,19 +722,9 @@ sub mount_static {
   (my MY $self, my ($location, $realpath)) = @_;
   my $app = ref $realpath eq 'CODE' ? $realpath
     : $self->psgi_file_app($realpath);
-  $self->mount_psgi
-    ($location, sub {
-       (my Env $env) = @_;
-       local $env->{PATH_INFO} = _trim_prefix($env->{PATH_INFO}, $location);
-       $app->($env);
-     });
+  $self->mount_psgi($location, $app);
 }
 
-sub _trim_prefix {
-  substr($_[0], length($_[1]));
-}
-
-
 #========================================
 
 sub mount_action {
@@ -566,11 +754,16 @@ sub Connection () {'YATT::Lite::Connection'};
 
 sub make_simple_connection {
   (my MY $self, my ($quad, @rest)) = @_;
-  my ($tmpldir, $loc, $file, $trailer) = @$quad;
+  my @params = $self->pi_to_connection_quad($quad);
+  $self->make_connection(undef, @params, @rest);
+}
+
+sub pi_to_connection_quad {
+  (my MY $self, my ($pi)) = @_;
+  my ($tmpldir, $loc, $file, $trailer) = @$pi;
   my $virtdir = "$self->{cf_doc_root}$loc";
   my $realdir = "$tmpldir$loc";
-  my @params = $self->connection_quad([$virtdir, $loc, $file, $trailer]);
-  $self->make_connection(undef, @params, @rest);
+  $self->connection_quad([$virtdir, $loc, $file, $trailer]);
 }
 
 sub make_debug_params {
@@ -608,7 +801,7 @@ sub run_dirhandler {
   (my MY $self, my ($dh, $con, $file)) = @_;
   local ($SYS, $YATT, $CON) = ($self, $dh, $con);
   $self->before_dirhandler($dh, $con, $file);
-  $self->invoke_dirhandler($dh, $con
+  $self->invoke_dirhandler($dh,
 			   , handle => $dh->cut_ext($file), $con, $file);
   $self->after_dirhandler($dh, $con, $file);
 }
@@ -617,10 +810,36 @@ sub before_dirhandler { &maybe::next::method; }
 sub after_dirhandler  { &maybe::next::method; }
 
 sub invoke_dirhandler {
-  (my MY $self, my ($dh, $con, $method, @args)) = @_;
+  (my MY $self, my ($dh, $method, @args)) = @_;
   $dh->with_system($self, $method, @args);
 }
 
+sub invoke_sub_in {
+  (my MY $self, my ($reqrec, $args, $sub, @rest)) = @_;
+
+  my $wantarray = wantarray;
+
+  my ($dh, $con, $widgetSpec)
+    = $self->prepare_processing_context($reqrec, $args);
+
+  local ($SYS, $YATT, $CON) = ($self, $dh, $con);
+
+  $self->before_dirhandler($dh, $con, lexpand($widgetSpec));
+
+  my @result;
+  if ($wantarray) {
+    @result = $sub->($dh, $con, @rest);
+  } else {
+    $result[0] = $sub->($dh, $con, @rest);
+  }
+
+  $self->after_dirhandler($dh, $con, lexpand($widgetSpec));
+
+  YATT::Lite::Util::try_invoke($con, 'flush_headers');
+
+  $wantarray ? @result : $result[0];
+}
+
 #========================================
 
 sub get_lochandler {
@@ -662,6 +881,9 @@ sub load_yatt {
   if (my $yatt = $self->{path2yatt}{$path}) {
     return $yatt;
   }
+  unless (-e $path) {
+    croak "Can't load YATT directory '$path'! No such directory!";
+  }
   if (not $visits) {
     $visits = Visits->start($path);
   } elsif (my $preds = $visits->check_cycle($path, $from)) {
@@ -672,10 +894,7 @@ sub load_yatt {
   if (not $self->{cf_allow_missing_dir} and not -d $path) {
     croak "Can't find '$path'!";
   }
-  if (my (@cf) = map {
-    my $cf = untaint_any($path) . "/.htyattconfig.$_";
-    -e $cf ? $cf : ()
-  } $self->config_filetypes) {
+  if (my (@cf) = $self->list_config_files(untaint_any($path)."/.htyattconfig")) {
     $self->error("Multiple configuration files!", @cf) if @cf > 1;
     _with_loading_file {$self} $cf[0], sub {
       $self->build_yatt($path, $basedir, $visits, $self->read_file($cf[0]));
@@ -707,6 +926,8 @@ sub build_yatt {
   if ($has_rc = (-e (my $rc = "$path/.htyattrc.pl"))) {
     # Note: This can do "use fields (...)"
     dofile_in($app_ns, $rc);
+
+    print STDERR "# Loaded: $rc\n" if DEBUG_FACTORY;
   }
 
   my @args = (vfs => [dir => $path
@@ -733,7 +954,12 @@ sub build_yatt {
 
   my $yatt = $self->{path2yatt}{$path} = $app_ns->new(@args, %opts);
 
+  unless ($yatt->after_new_is_called) {
+    Carp::croak("after_new is not called for $path!");
+  }
+
   if ($has_rc) {
+    print STDERR "# setting up rc actions\n" if DEBUG_FACTORY;
     $yatt->setup_rc_actions;
   }
 
@@ -860,6 +1086,7 @@ sub _cf_delegates {
      output_encoding
      header_charset
      tmpl_encoding
+     render_as_bytes
      debug_cgen
      at_done
      app_root
@@ -881,6 +1108,10 @@ sub _cf_delegates {
      debug_parser
      check_lineno
      match_argsroute_first
+     stash_unknown_params_to
+     body_argument
+     body_argument_type
+     prefer_call_for_entity
   );
 }
 
@@ -888,11 +1119,18 @@ sub configparams_for {
   (my MY $self, my $hash) = @_;
   # my @base = map { [dir => $_] } lexpand($self->{cf_tmpldirs});
   # (@base ? (base => \@base) : ())
+
+  my $debugging = YATT::Lite::Util::is_debugging();
+  if ($debugging) {
+    print STDERR "# Note: Factory->die_in_error is turned off for debugging\n";
+  }
+
   (
    $self->cf_delegate_known(0, $hash, $self->_cf_delegates)
    , (exists $hash->{cf_error_handler}
       ? (error_handler => \ $self->{cf_error_handler}) : ())
-   , die_in_error => ! YATT::Lite::Util::is_debugging());
+   , die_in_error => ! $debugging
+ );
 }
 
 # XXX: Should have better interface.
@@ -945,13 +1183,30 @@ sub read_file {
 sub default_config_filetypes {qw/xhf yml/}
 sub config_filetypes {
   (my MY $self) = @_;
-  if (my $item = $self->{cf_config_filetypes}) {
+  if (ref $self and my $item = $self->{cf_config_filetypes}) {
     lexpand($item)
   } else {
     $self->default_config_filetypes
   }
 }
 
+sub find_unique_config_file {
+  (my MY $self, my ($base_path, @rest)) = @_;
+  return if grep {not defined} $base_path, @rest;
+  my @cf = $self->list_config_files(join("", $base_path, @rest));
+  $self->error("Multiple configuration files!", @cf) if @cf > 1;
+  return $cf[0] if @cf;
+  return;
+}
+
+sub list_config_files {
+  (my MY $self, my $base_path) = @_;
+  map {
+    my $cf = "$base_path.$_";
+    -e $cf ? $cf : ()
+  } $self->config_filetypes
+}
+
 sub read_file_xhf {
   (my MY $self, my $fn) = @_;
   my $bytes_semantics = ref $self && $self->{cf_binary_config};
@@ -963,7 +1218,7 @@ sub read_file_yml {
   (my MY $self, my $fn) = @_;
   require YAML::Tiny;
   my $yaml = YAML::Tiny->read($fn);
-  wantarray ? lexpand($yaml->[0]) : $yaml;
+  wantarray ? lexpand($yaml->[0]) : $yaml->[0];
 }
 
 #========================================
