@@ -9,10 +9,13 @@ our @EXPORT_OK = @EXPORT;
 
 use YATT::Lite::Util qw/ckeval globref/;
 
+use constant DEBUG_IMPORT => $ENV{DEBUG_YATT_IMPORT} // 0;
+
 require mro;
 require YATT::Lite::MFields;
 
 sub import {
+  Carp::carp(scalar caller, " calls $_[0]->import()") if DEBUG_IMPORT;
   parse_args(\@_, scalar caller);
   goto &Exporter::import;
 }
@@ -46,13 +49,9 @@ sub _import_as_base {
     unless ($isa = *{$sym}{ARRAY}) {
       *$sym = $isa = [];
     }
-    my $using_c3 = mro::get_mro($callpack) eq 'c3';
+    mro::set_mro($callpack, 'c3');
     unless (grep {$_ eq $myPack} @$isa) {
-      if ($using_c3) {
-	unshift @$isa, $myPack;
-      } else {
-	push @$isa, $myPack;
-      }
+      unshift @$isa, $myPack;
     }
   }
 
